@@ -18,10 +18,10 @@ class KLdapTest : public QObject
     Q_OBJECT
 public:
     explicit KLdapTest(QObject *parent = nullptr);
-    ~KLdapTest() = default;
+    ~KLdapTest() override = default;
 
 private Q_SLOTS:
-    //void testKLdap();
+    // void testKLdap();
 
     void initTestCase();
     void cleanupTestCase();
