@@ -4,14 +4,15 @@
  *
  * Ported to KABC by Daniel Molkentin <molkentin@kde.org>
  *
- * SPDX-FileCopyrightText: 2013-2020 Laurent Montel <montel@kde.org>
+ * SPDX-FileCopyrightText: 2013-2021 Laurent Montel <montel@kde.org>
  *
  * SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
 #include "ldapclientsearch.h"
-#include "ldapclientsearchconfig.h"
 #include "ldapclient_debug.h"
+#include "ldapclientsearchconfig.h"
+#include "ldapsearchclientreadconfigserverjob.h"
 
 #include "ldapclient.h"
 
@@ -27,8 +28,8 @@
 
 #include <kio/job.h>
 
-#include <QTimer>
 #include <QStandardPaths>
+#include <QTimer>
 
 using namespace KLDAP;
 
@@ -37,15 +38,11 @@ class Q_DECL_HIDDEN LdapClientSearch::Private
 public:
     Private(LdapClientSearch *qq)
         : q(qq)
-        , mActiveClients(0)
-        , mNoLDAPLookup(false)
     {
-        mClientSearchConfig = new LdapClientSearchConfig;
     }
 
     ~Private()
     {
-        delete mClientSearchConfig;
     }
 
     void readWeighForClient(LdapClient *client, const KConfigGroup &config, int clientNumber);
@@ -58,8 +55,9 @@ public:
     void slotLDAPDone();
     void slotDataTimer();
     void slotFileChanged(const QString &);
+    void init(const QStringList &attributes);
 
-    LdapClientSearch *q = nullptr;
+    LdapClientSearch *const q;
     QList<LdapClient *> mClients;
     QStringList mAttributes;
     QString mSearchText;
@@ -69,35 +67,20 @@ public:
     bool mNoLDAPLookup = false;
     LdapResultObject::List mResults;
     QString mConfigFile;
-    LdapClientSearchConfig *mClientSearchConfig = nullptr;
 };
 
 LdapClientSearch::LdapClientSearch(QObject *parent)
     : QObject(parent)
     , d(new Private(this))
 {
-    Kdelibs4ConfigMigrator migrate(QStringLiteral("ldapsettings"));
-    migrate.setConfigFiles(QStringList() << QStringLiteral("kabldaprc"));
-    migrate.migrate();
+    d->init(LdapClientSearch::defaultAttributes());
+}
 
-    if (!KProtocolInfo::isKnownProtocol(QUrl(QStringLiteral("ldap://localhost")))) {
-        d->mNoLDAPLookup = true;
-        return;
-    }
-
-    d->mAttributes << QStringLiteral("cn")
-                   << QStringLiteral("mail")
-                   << QStringLiteral("givenname")
-                   << QStringLiteral("sn");
-
-    // Set the filter, to make sure old usage (before 4.14) of this object still works.
-    d->mFilter = QStringLiteral("&(|(objectclass=person)(objectclass=groupOfNames)(mail=*))"
-                                "(|(cn=%1*)(mail=%1*)(givenName=%1*)(sn=%1*))");
-
-    d->readConfig();
-    connect(KDirWatch::self(), &KDirWatch::dirty, this, [this](const QString &filename) {
-        d->slotFileChanged(filename);
-    });
+LdapClientSearch::LdapClientSearch(const QStringList &attr, QObject *parent)
+    : QObject(parent)
+    , d(new Private(this))
+{
+    d->init(attr);
 }
 
 LdapClientSearch::~LdapClientSearch()
@@ -105,6 +88,30 @@ LdapClientSearch::~LdapClientSearch()
     delete d;
 }
 
+void LdapClientSearch::Private::init(const QStringList &attributes)
+{
+    Kdelibs4ConfigMigrator migrate(QStringLiteral("ldapsettings"));
+    migrate.setConfigFiles(QStringList() << QStringLiteral("kabldaprc"));
+    migrate.migrate();
+
+    if (!KProtocolInfo::isKnownProtocol(QUrl(QStringLiteral("ldap://localhost")))) {
+        mNoLDAPLookup = true;
+        return;
+    }
+
+    mAttributes = attributes;
+
+    // Set the filter, to make sure old usage (before 4.14) of this object still works.
+    mFilter = QStringLiteral(
+        "&(|(objectclass=person)(objectclass=groupOfNames)(mail=*))"
+        "(|(cn=%1*)(mail=%1*)(givenName=%1*)(sn=%1*))");
+
+    readConfig();
+    q->connect(KDirWatch::self(), &KDirWatch::dirty, q, [this](const QString &filename) {
+        slotFileChanged(filename);
+    });
+}
+
 void LdapClientSearch::Private::readWeighForClient(LdapClient *client, const KConfigGroup &config, int clientNumber)
 {
     const int completionWeight = config.readEntry(QStringLiteral("SelectedCompletionWeight%1").arg(clientNumber), -1);
@@ -116,8 +123,8 @@ void LdapClientSearch::Private::readWeighForClient(LdapClient *client, const KCo
 void LdapClientSearch::updateCompletionWeights()
 {
     KConfigGroup config(KLDAP::LdapClientSearchConfig::config(), "LDAP");
-    for (int i = 0; i < d->mClients.size(); ++i) {
-        d->readWeighForClient(d->mClients[ i ], config, i);
+    for (int i = 0, total = d->mClients.size(); i < total; ++i) {
+        d->readWeighForClient(d->mClients[i], config, i);
     }
 }
 
@@ -149,6 +156,12 @@ void LdapClientSearch::setAttributes(const QStringList &attrs)
     }
 }
 
+QStringList LdapClientSearch::defaultAttributes()
+{
+    const QStringList attr{QStringLiteral("cn"), QStringLiteral("mail"), QStringLiteral("givenname"), QStringLiteral("sn")};
+    return attr;
+}
+
 void LdapClientSearch::Private::readConfig()
 {
     q->cancelSearch();
@@ -162,28 +175,26 @@ void LdapClientSearch::Private::readConfig()
         mNoLDAPLookup = true;
     } else {
         for (int j = 0; j < numHosts; ++j) {
-            auto *ldapClient = new LdapClient(j, q);
-            KLDAP::LdapServer server;
-            mClientSearchConfig->readConfig(server, config, j, true);
-            if (!server.host().isEmpty()) {
-                mNoLDAPLookup = false;
-            }
-            ldapClient->setServer(server);
+            auto ldapClient = new LdapClient(j, q);
+            auto job = new LdapSearchClientReadConfigServerJob;
+            job->setCurrentIndex(j);
+            job->setActive(true);
+            job->setConfig(config);
+            job->setLdapClient(ldapClient);
+            job->start();
 
+            mNoLDAPLookup = false;
             readWeighForClient(ldapClient, config, j);
 
             ldapClient->setAttributes(mAttributes);
 
-            q->connect(ldapClient, &LdapClient::result,
-                       q, [this](const LdapClient &client, const KLDAP::LdapObject &obj) {
+            q->connect(ldapClient, &LdapClient::result, q, [this](const LdapClient &client, const KLDAP::LdapObject &obj) {
                 slotLDAPResult(client, obj);
             });
-            q->connect(ldapClient, &LdapClient::done,
-                       q, [this]() {
+            q->connect(ldapClient, &LdapClient::done, q, [this]() {
                 slotLDAPDone();
             });
-            q->connect(ldapClient, qOverload<const QString &>(&LdapClient::error),
-                       q, [this](const QString &str) {
+            q->connect(ldapClient, qOverload<const QString &>(&LdapClient::error), q, [this](const QString &str) {
                 slotLDAPError(str);
             });
 
@@ -312,18 +323,17 @@ void LdapClientSearch::Private::makeSearchData(QStringList &ret, LdapResult::Lis
         bool wasCN = false;
         bool wasDC = false;
 
-        //qCDebug(LDAPCLIENT_LOG) <<"\n\nLdapClientSearch::makeSearchData()";
+        // qCDebug(LDAPCLIENT_LOG) <<"\n\nLdapClientSearch::makeSearchData()";
 
         KLDAP::LdapAttrMap::ConstIterator it2;
-        for (it2 = (*it1).object.attributes().constBegin();
-             it2 != (*it1).object.attributes().constEnd(); ++it2) {
+        for (it2 = (*it1).object.attributes().constBegin(); it2 != (*it1).object.attributes().constEnd(); ++it2) {
             QByteArray val = (*it2).first();
             int len = val.size();
             if (len > 0 && '\0' == val[len - 1]) {
                 --len;
             }
             const QString tmp = QString::fromUtf8(val.constData(), len);
-            //qCDebug(LDAPCLIENT_LOG) <<"      key: \"" << it2.key() <<"\" value: \"" << tmp <<"\"";
+            // qCDebug(LDAPCLIENT_LOG) <<"      key: \"" << it2.key() <<"\" value: \"" << tmp <<"\"";
             if (it2.key() == QLatin1String("cn")) {
                 name = tmp;
                 if (mail.isEmpty()) {
@@ -359,8 +369,7 @@ void LdapClientSearch::Private::makeSearchData(QStringList &ret, LdapResult::Lis
                 givenname = tmp;
             } else if (it2.key() == QLatin1String("sn")) {
                 sn = tmp;
-            } else if (it2.key() == QLatin1String("objectClass")
-                       && (tmp == QLatin1String("groupOfNames") || tmp == QLatin1String("kolabGroupOfNames"))) {
+            } else if (it2.key() == QLatin1String("objectClass") && (tmp == QLatin1String("groupOfNames") || tmp == QLatin1String("kolabGroupOfNames"))) {
                 isDistributionList = true;
             }
         }
@@ -370,7 +379,7 @@ void LdapClientSearch::Private::makeSearchData(QStringList &ret, LdapResult::Lis
                 mails.append(mail);
             }
             if (isDistributionList) {
-                //qCDebug(LDAPCLIENT_LOG) <<"\n\nLdapClientSearch::makeSearchData() found a list:" << name;
+                // qCDebug(LDAPCLIENT_LOG) <<"\n\nLdapClientSearch::makeSearchData() found a list:" << name;
                 ret.append(name);
                 // following lines commented out for bugfixing kolab issue #177:
                 //
@@ -379,13 +388,13 @@ void LdapClientSearch::Private::makeSearchData(QStringList &ret, LdapResult::Lis
                 // The right server is found by the SMTP server instead: Kolab users
                 // must use the correct SMTP server, by definition.
                 //
-                //mail = (*it1).client->base().simplified();
-                //mail.replace( ",dc=", ".", false );
-                //if( mail.startsWith("dc=", false) )
+                // mail = (*it1).client->base().simplified();
+                // mail.replace( ",dc=", ".", false );
+                // if( mail.startsWith("dc=", false) )
                 //  mail.remove(0, 3);
-                //mail.prepend( '@' );
-                //mail.prepend( name );
-                //mail = name;
+                // mail.prepend( '@' );
+                // mail.prepend( name );
+                // mail = name;
             } else {
                 continue; // nothing, bad entry
             }
