@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: 2019-2020 Laurent Montel <montel@kde.org>
+ * SPDX-FileCopyrightText: 2019-2021 Laurent Montel <montel@kde.org>
  *
  * SPDX-License-Identifier: LGPL-2.0-or-later
  */
@@ -7,7 +7,6 @@
 #ifndef LDAPCONFIGUREWIDGET_H
 #define LDAPCONFIGUREWIDGET_H
 
-
 #include "kldap_export.h"
 
 #include <QWidget>
@@ -17,7 +16,8 @@ class QPushButton;
 class QToolButton;
 class QListWidgetItem;
 
-namespace KLDAP {
+namespace KLDAP
+{
 class LdapClientSearchConfig;
 /**
  * @brief The LdapConfigureWidget class
